import asyncio
from functools import partial
import os
import boto3
from botocore.exceptions import ClientError

class S3Service:
    def __init__(self, aws_access_key_id, aws_secret_access_key, region_name, bucket_name, bucket_path):
        self.s3 = boto3.client(
            "s3",
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name,
        )
        
        self.bucket_name = bucket_name
        self.bucket_path = bucket_path

        self.loop = asyncio.get_running_loop()

    async def save_on_s3(self, file_path, file_name):
        try:
            await self.loop.run_in_executor(
                None,
                self.s3.upload_file,
                file_path,
                self.bucket_name,
                self.bucket_path + file_name,
            )
        except Exception as e:
            print(f"❌ Erro ao salvar o arquivo no S3: {e}")

    async def save_many_paths_on_s3(self, file_paths):
        """
        Salva vários arquivos no S3 informando apenas os caminhos locais.
        O nome do arquivo no S3 será o mesmo do arquivo local.
        """
        tasks = [
            self.save_on_s3(
                file_path["caminho_arquivo_local"], file_path["nome_arquivo"]
            )
            for file_path in file_paths
        ]
        await asyncio.gather(*tasks)

    async def file_on_s3(self, file_name):
        try:
            head_object_func = partial(
                self.s3.head_object,
                Bucket=self.bucket_name,
                Key=self.bucket_path + file_name,
            )

            await self.loop.run_in_executor(None, head_object_func)
            return True
        except ClientError as e:
            if e.response["Error"]["Code"] == "404":
                return False

            raise
        except Exception as e:
            print(f"❌ Erro ao obter o arquivo do S3: {e}")
            return False

    async def get_file_from_s3(self, file_name, destination_path):
        try:
            os.makedirs(os.path.dirname(destination_path), exist_ok=True)

            key = self.bucket_path + file_name
            print(f"🔍 Verificando chave: {key}")
            download_func = partial(
                self.s3.download_file, self.bucket_name, key, destination_path
            )

            await self.loop.run_in_executor(None, download_func)
            print(f"📖✅ Arquivo '{file_name}' baixado com sucesso.")
        except ClientError as e:
            if e.response["Error"]["Code"] == "404":
                print(f"⚠️ Arquivo '{file_name}' não encontrado no S3.")
            else:
                print(f"❌ Erro ao baixar o arquivo do S3: {e}")
        except Exception as e:
            print(f"❌ Erro inesperado ao baixar o arquivo do S3: {e}")



from enum import Enum


class EFaseRetificacao(Enum):
    NaoIniciado = 0
    SolicitacaoXml = 1
    AguardandoXml = 2
    DownloadXml = 3
    ExtraindoDadosDoXml = 4
    #? Abertura de Competencia
    EstruturandoXmlAberturaCompetencia = 5
    AberturaDeCompetencia = 6
    ConsultandoESocialAberturaCompetencia = 7
    #? Rubricas
    EstruturandoXmlInclusaoRubricas = 8
    InclusaoDasRubricas = 9
    ConsultandoESocialInclusaoRubricas = 10
    #? Exclusao de Pagamentos
    EstruturandoXmlExclusaoPagamentos = 11
    ExclusaoDePagamentos = 12
    ConsultandoESocialExclusaoPagamentos = 13
    #? Retificacao
    EstruturandoXmlRetificacaoRemuneracao = 14
    RetificacaoDaRemuneracao = 15
    ConsultandoESocialRetificacaoRemuneracao = 16
    #? Desligamento
    EstruturandoXmlDesligamento = 17
    Desligamento = 18
    ConsultandoESocialDesligamento = 19
    #? Inclusao de Pagamentos
    EstruturandoXmlInclusaoPagamentos = 20
    InclusaoDosPagamentos = 21
    ConsultandoESocialInclusaoPagamentos = 22
    #? Fechamento de Competencia
    EstruturandoXmlFechamentoCompetencia = 23
    FechamentoDeCompetencia = 24
    ConsultandoESocialFechamentoCompetencia = 25
    Finalizado = 26



import asyncio
from io import BytesIO
import os
import shutil
from pathlib import Path
import zipfile
import aiofiles
import aiofiles.os

class FileService:
    async def zip_folder(self, folder_path: str, output_path: str):
        buffer = BytesIO()
        
        with zipfile.ZipFile(buffer, "w", zipfile.ZIP_DEFLATED) as zip_file:
            for root, _, files in os.walk(folder_path):
                for file in files:
                    full_path = os.path.join(root, file)
                    arcname = os.path.relpath(full_path, folder_path)

                    async with aiofiles.open(full_path, "rb") as f:
                        content = await f.read()
                        zip_file.writestr(arcname, content)
        
        async with aiofiles.open(output_path, "wb") as f:
            await f.write(buffer.getvalue())
            
    async def extract_zip(self, folder_path_zip: str, output_path: str):
            if not await aiofiles.os.path.exists(folder_path_zip):
                raise FileNotFoundError(
                    f"Arquivo ZIP não encontrado: {folder_path_zip}"
                )

            if not await aiofiles.os.path.exists(output_path):
                await aiofiles.os.makedirs(output_path, exist_ok=True)

            loop = asyncio.get_event_loop()
            await loop.run_in_executor(
                None, self._extrair_zip, folder_path_zip, output_path
            )

    async def remove_file(self, file_path: str):
        if await aiofiles.os.path.exists(file_path):
            await aiofiles.os.remove(file_path)
            
    async def remove_folder(self, folder_path: str):
        if await aiofiles.os.path.exists(folder_path):
            loop = asyncio.get_running_loop()
            await loop.run_in_executor(None, shutil.rmtree, folder_path)
            
    def _extrair_zip(self, folder_path_zip: str, output_path: str):
        with zipfile.ZipFile(folder_path_zip, "r") as zip_ref:
            zip_ref.extractall(output_path)
            


import httpx


class RequestServicoApi:
    def __init__(self, url, token):
        self.url = url
        self.token = token
        self.client = httpx.AsyncClient(timeout=1200, verify=False)
        self.status_code_success = [200, 201, 204]

    async def post(self, *, data: dict | list):
        print(self.token)
        response = await self.client.post(self.url, json=data)

        if response.status_code not in self.status_code_success:
            raise Exception(f"Erro ao fazer request ao servico de API: {response.status_code}")
        
        await self.close()

    async def get(self):
        response = await self.client.get(self.url)

        if response.status_code not in self.status_code_success:
            raise Exception(f"Erro ao fazer request ao servico de API: {response.status_code}")

        await self.close()
        return response.json()

    async def close(self):
        await self.client.aclose()


import uuid

class DepententeInvalido1210Repository:
    def __init__(self, db):
        self.__db = db

    async def inserir(self, depentente_invalido: dict) -> bool:
        
        try:
            dep_ja_cadastrado = await self.__db.dependente_invalido_1210.find_one({"solicitacaoId": depentente_invalido["solicitacaoId"], "cpf": depentente_invalido["cpf"], "cpf_dep": depentente_invalido["cpf_dep"], "per_apur": depentente_invalido["per_apur"]})
            if dep_ja_cadastrado:
                print(f"⚠️ Dependente inválido 1210 já cadastrado: {dep_ja_cadastrado}")
                return False

            await self.__db.dependente_invalido_1210.insert_one(depentente_invalido)
            return True
        except Exception as e:
            print(f"❌ Erro ao inserir o dependente inválido 1210: {e}")
            return False
        
    async def buscar_por_solicitacao_id(self, solicitacaoId: int) -> list:
        try:
            return await self.__db.dependente_invalido_1210.find({"solicitacaoId": solicitacaoId}, {"_id": 0}).to_list(length=None)
        except Exception as e:
            print(f"❌ Erro ao buscar dependente inválido 1210 por solicitacaoId: {e}")
            return []
    
    async def remover(self, solicitacaoId: int, cpf_colaborador: str) -> bool:
        try:
            await self.__db.dependente_invalido_1210.delete_many({"solicitacaoId": solicitacaoId, "cpf": cpf_colaborador})
            return True
        except Exception as e:
            print(f"❌ Erro ao remover dependente inválido 1210: {e}")
            return False
